=== modified file 'Makefile'
--- Makefile	2009-01-06 05:10:34 +0000
+++ Makefile	2009-01-17 02:12:05 +0000
@@ -9,13 +9,14 @@
 #DEBUG=-ggdb3
 # For info about _FORTIFY_SOURCE, see
 # <http://gcc.gnu.org/ml/gcc-patches/2004-09/msg02055.html>
-FORTIFY=-D_FORTIFY_SOURCE=2 -fstack-protector-all -fPIE -pie
-LINK_FORTIFY=-z relro -pie
+FORTIFY=-D_FORTIFY_SOURCE=2 -fstack-protector-all -fPIC -fPIE
+LINK_FORTIFY_LD=-z relro -fPIE
+LINK_FORTIFY=-pie
 #COVERAGE=--coverage
 OPTIMIZE=-Os
 LANGUAGE=-std=gnu99
 htmldir=man
-version=1.0.3
+version=1.0.5
 SED=sed
 
 ## Use these settings for a traditional /usr/local install
@@ -45,7 +46,7 @@
 CFLAGS=$(WARN) $(DEBUG) $(FORTIFY) $(COVERAGE) $(OPTIMIZE) \
 	$(LANGUAGE) $(GNUTLS_CFLAGS) $(AVAHI_CFLAGS) $(GPGME_CFLAGS) \
 	-DVERSION='"$(version)"'
-LDFLAGS=$(COVERAGE) $(foreach flag,$(LINK_FORTIFY),-Xlinker $(flag))
+LDFLAGS=$(COVERAGE) $(LINK_FORTIFY) $(foreach flag,$(LINK_FORTIFY_LD),-Xlinker $(flag))
 
 # Commands to format a DocBook <refentry> document into a manual page
 DOCBOOKTOMAN=cd $(dir $<); xsltproc --nonet --xinclude \
@@ -76,7 +77,7 @@
 PLUGINS=plugins.d/password-prompt plugins.d/mandos-client \
 	plugins.d/usplash plugins.d/splashy plugins.d/askpass-fifo
 CPROGS=plugin-runner $(PLUGINS)
-PROGS=mandos mandos-keygen $(CPROGS)
+PROGS=mandos mandos-keygen mandos-ctl $(CPROGS)
 DOCS=mandos.8 plugin-runner.8mandos mandos-keygen.8 \
 	plugins.d/mandos-client.8mandos \
 	plugins.d/password-prompt.8mandos mandos.conf.5 \
@@ -163,6 +164,11 @@
 		--expression='s/^\(VERSION="\)[^"]*"$$/\1$(version)"/' \
 		$@
 
+mandos-ctl: Makefile
+	$(SED) --in-place \
+		--expression='s/^\(version = "\)[^"]*"$$/\1$(version)"/' \
+		$@
+
 mandos.lsm: Makefile
 	$(SED) --in-place \
 		--expression='s/^\(Version:\).*/\1\t$(version)/' \
@@ -170,6 +176,9 @@
 	$(SED) --in-place \
 		--expression='s/^\(Entered-date:\).*/\1\t$(shell date --rfc-3339=date --reference=Makefile)/' \
 		$@
+	$(SED) --in-place \
+		--expression='s/\(mandos_\)[0-9.]\+\(\.orig\.tar\.gz\)/\1$(version)\2/' \
+		$@
 
 plugins.d/mandos-client: plugins.d/mandos-client.o
 	$(LINK.o) $(GNUTLS_LIBS) $(AVAHI_LIBS) $(GPGME_LIBS) \
@@ -194,7 +203,8 @@
 run-client: all keydir/seckey.txt keydir/pubkey.txt
 	./plugin-runner --plugin-dir=plugins.d \
 		--config-file=plugin-runner.conf \
-		--options-for=mandos-client:--seckey=keydir/seckey.txt,--pubkey=keydir/pubkey.txt
+		--options-for=mandos-client:--seckey=keydir/seckey.txt,--pubkey=keydir/pubkey.txt \
+		$(CLIENTARGS)
 
 # Used by run-client
 keydir/seckey.txt keydir/pubkey.txt: mandos-keygen
@@ -203,7 +213,7 @@
 
 # Run the server with a local config
 run-server: confdir/mandos.conf confdir/clients.conf
-	./mandos --debug --configdir=confdir
+	./mandos --debug --configdir=confdir $(SERVERARGS)
 
 # Used by run-server
 confdir/mandos.conf: mandos.conf

=== modified file 'NEWS'
--- NEWS	2009-01-06 05:10:34 +0000
+++ NEWS	2009-01-17 02:12:05 +0000
@@ -1,6 +1,22 @@
 This NEWS file records noteworthy changes, very tersely.
 See the manual for detailed information.
 
+Version 1.0.5 (2009-01-17)
+* Client
+** Fix small memory leak in plugin-runner.
+
+Version 1.0.4 (2009-01-15)
+* Server
+** Only find matched user/group pairs when searching for suitable
+   nonprivileged user/group to switch to.
+
+* Client
+** New kernel parameter "mandos=off" makes client not run at boot.
+** Fix linking errors and compilation warnings on AMD64.
+** Parse numbers in command line options better.
+** The splashy and usplash plugins are more robust while traversing
+   /proc, and will not abort if a process suddenly disappears.
+
 Version 1.0.3 (2009-01-06)
 * Server
 ** Now tries to change to user and group "_mandos" before falling back

=== modified file 'README'
--- README	2009-01-04 21:54:55 +0000
+++ README	2009-01-12 22:02:33 +0000
@@ -152,8 +152,6 @@
     cryptsetup, this plugin listens to the same FIFO as askpass would
     do.
   
-  (None of these take any options or reads any files.)
-  
   More plugins could easily be written and added by the system
   administrator; see the section called "WRITING PLUGINS" in
   plugin-runner(8mandos) to learn the plugin requirements.

=== modified file 'TODO'
--- TODO	2008-12-29 02:44:54 +0000
+++ TODO	2009-01-18 00:16:57 +0000
@@ -53,12 +53,12 @@
 ** TODO [#B] "--test" option
    For testing decryption before rebooting.
 
-* [#A] Package
+* Package
 ** /usr/share/initramfs-tools/hooks/mandos
-*** TODO Do not install in initrd.img if configured not to.
+*** TODO [#C] Do not install in initrd.img if configured not to.
     Use "/etc/initramfs-tools/conf.d/mandos"?  Definitely a debconf
     question.
-** TODO /etc/bash_completion.d/mandos
+** TODO [#C] /etc/bash_completion.d/mandos
    From XML sources directly?
 
 

=== modified file 'clients.conf'
--- clients.conf	2008-09-21 13:42:34 +0000
+++ clients.conf	2009-01-08 03:54:06 +0000
@@ -14,7 +14,7 @@
 ;interval = 5m
 
 # What command to run as "the checker".
-;checker = fping -q -- %(host)s
+;checker = fping -q -- %%(host)s
 
 
 ;####

=== modified file 'debian/changelog'
--- debian/changelog	2009-01-06 20:22:18 +0000
+++ debian/changelog	2009-01-17 02:12:05 +0000
@@ -1,11 +1,32 @@
+mandos (1.0.5-1) unstable; urgency=low
+
+  * New upstream release.
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>  Sat, 17 Jan 2009 02:26:00 +0100
+
+mandos (1.0.4-1) unstable; urgency=low
+
+  * New upstream release.
+  * debian/watch: New file.
+  * debian/mandos-client.README.Debian: Document new "mandos=off" kernel
+    parameter.
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>  Thu, 15 Jan 2009 05:49:22 +0100
+
+mandos (1.0.3-2) unstable; urgency=low
+
+  * Removed some now-unused debconf files.
+  * Changed postinst scripts to not source debconf/confmodule.
+  * Removed po-debconf from build-depends.
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>  Tue, 06 Jan 2009 21:28:20 +0100
+
 mandos (1.0.3-1) unstable; urgency=low
 
   * New upstream release.
-    
   * Add -Xlinker to linker flags to fix FTBFS for some architectures.
     Thanks to Thiemo Seufer <ths@networkno.de> for the report and
     fix. (Closes:  #509398)
-    
   * Remove debconf use altogether, thereby stopping debconf abuse.  Thanks
     to Christian Perrier <bubulle@debian.org>. (Closes:  #509653)
   * Add NEWS file to /usr/share/doc directories.

=== modified file 'debian/mandos-client.README.Debian'
--- debian/mandos-client.README.Debian	2008-10-05 17:38:31 +0000
+++ debian/mandos-client.README.Debian	2009-01-15 02:52:02 +0000
@@ -42,4 +42,10 @@
   Mandos client will be the new default way for getting a password for
   the root file system when booting.
 
- -- Teddy Hogeborn <teddy@fukt.bsnet.se>, Sun,  5 Oct 2008 19:04:26 +0200
+* Emergency Escape
+  
+  If it ever should be necessary, the Mandos client can be temporarily
+  prevented from running at startup by passing the parameter
+  "mandos=off" to the kernel.
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>, Mon, 12 Jan 2009 02:29:10 +0100

=== modified file 'debian/mandos-client.lintian-overrides'
--- debian/mandos-client.lintian-overrides	2008-10-01 15:29:01 +0000
+++ debian/mandos-client.lintian-overrides	2009-01-18 06:41:57 +0000
@@ -1,8 +1,3 @@
-# This example command line is long without spaces, but it must be
-# that way; it's part of the point of showing it.
-#
-mandos-client binary: manpage-has-errors-from-man usr/share/man/man8/plugin-runner.8mandos.gz 297: warning [p 4, 5.8i]: can't break line
-
 # This directory contains secret client key files.
 #
 mandos-client binary: non-standard-dir-perm etc/keys/mandos/ 0700 != 0755
@@ -16,7 +11,7 @@
 
 # These binaries must be setuid root, since they need root powers, but
 # are started by plugin-runner(8mandos), which runs all plugins as
-# user/group "mandos".  These binaries are not run in a running
+# user/group "_mandos".  These binaries are not run in a running
 # system, but in an initial RAM disk environment.  Here they are
 # protected from non-root access by the directory permissions, above.
 #

=== modified file 'debian/mandos-client.postinst'
--- debian/mandos-client.postinst	2009-01-06 05:08:37 +0000
+++ debian/mandos-client.postinst	2009-01-18 00:16:57 +0000
@@ -1,4 +1,4 @@
-#!/bin/bash -e
+#!/bin/sh -e
 # This script can be called in the following ways:
 #
 # After the package was installed:
@@ -26,7 +26,7 @@
 # Add user and group
 add_mandos_user(){
     # Rename old "mandos" user and group
-    case "$(getent passwd mandos)" in
+    case "`getent passwd mandos`" in
 	*:Mandos\ password\ system,,,:/nonexistent:/bin/false)
 	    usermod --login _mandos mandos
 	    groupmod --new-name _mandos mandos
@@ -62,7 +62,7 @@
 	;;
 
     *)
-	echo "$0 called with unknown argument \`$1'" 1>&2
+	echo "$0 called with unknown argument '$1'" 1>&2
 	exit 1
 	;;
 esac

=== modified file 'debian/mandos-client.postrm'
--- debian/mandos-client.postrm	2008-09-19 20:54:58 +0000
+++ debian/mandos-client.postrm	2009-01-18 00:16:57 +0000
@@ -50,7 +50,7 @@
 	;;
 
     *)
-	echo "$0 called with unknown argument \`$1'" 1>&2
+	echo "$0 called with unknown argument '$1'" 1>&2
 	exit 1
 	;;
 esac

=== modified file 'debian/mandos.postinst'
--- debian/mandos.postinst	2009-01-06 05:08:37 +0000
+++ debian/mandos.postinst	2009-01-18 00:16:57 +0000
@@ -1,4 +1,4 @@
-#!/bin/bash -e
+#!/bin/sh -e
 # This script can be called in the following ways:
 #
 # After the package was installed:
@@ -18,7 +18,7 @@
 case "$1" in
     configure)
 	# Rename old "mandos" user and group
-	case "$(getent passwd mandos)" in
+	case "`getent passwd mandos`" in
 	    *:Mandos\ password\ system,,,:/nonexistent:/bin/false)
 		usermod --login _mandos mandos
 		groupmod --new-name _mandos mandos
@@ -37,7 +37,7 @@
 	;;
 
     *)
-	echo "$0 called with unknown argument \`$1'" 1>&2
+	echo "$0 called with unknown argument '$1'" 1>&2
 	exit 1
 	;;
 esac

=== modified file 'debian/mandos.prerm'
--- debian/mandos.prerm	2008-09-21 13:42:34 +0000
+++ debian/mandos.prerm	2009-01-18 00:16:57 +0000
@@ -6,12 +6,12 @@
 set -e
 
 # summary of how this script can be called:
-#        * <prerm> `remove'
-#        * <old-prerm> `upgrade' <new-version>
-#        * <new-prerm> `failed-upgrade' <old-version>
-#        * <conflictor's-prerm> `remove' `in-favour' <package> <new-version>
-#        * <deconfigured's-prerm> `deconfigure' `in-favour'
-#          <package-being-installed> <version> `removing'
+#        * <prerm> 'remove'
+#        * <old-prerm> 'upgrade' <new-version>
+#        * <new-prerm> 'failed-upgrade' <old-version>
+#        * <conflictor's-prerm> 'remove' 'in-favour' <package> <new-version>
+#        * <deconfigured's-prerm> 'deconfigure' 'in-favour'
+#          <package-being-installed> <version> 'removing'
 #          <conflicting-package> <version>
 # for details, see /usr/share/doc/packaging-manual/
 
@@ -28,7 +28,7 @@
     upgrade|failed-upgrade)
         ;;
     *)
-        echo "prerm called with unknown argument \`$1'" >&2
+        echo "prerm called with unknown argument '$1'" >&2
         exit 0
     ;;
 esac

=== modified file 'debian/rules'
--- debian/rules	2009-01-04 22:26:07 +0000
+++ debian/rules	2009-01-18 00:18:50 +0000
@@ -47,7 +47,7 @@
 	dh_installdirs --indep
 	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos install-server
 	dh_lintian
-	dh_installinit --onlyscripts --no-start \
+	dh_installinit --onlyscripts \
 		--update-rcd-params="defaults 25 15"
 	dh_install --indep
 

=== added file 'debian/watch'
--- debian/watch	1970-01-01 00:00:00 +0000
+++ debian/watch	2009-01-15 02:52:02 +0000
@@ -0,0 +1,2 @@
+version=3
+ftp://ftp.fukt.bsnet.se/pub/mandos/mandos[-_]([^\s]+?)(?:\.orig)?\.tar\.(?:gz|bz2|7z)

=== modified file 'initramfs-tools-script'
--- initramfs-tools-script	2008-09-07 15:42:11 +0000
+++ initramfs-tools-script	2009-01-15 02:52:02 +0000
@@ -24,6 +24,12 @@
      ;;
 esac
 
+for param in `cat /proc/cmdline`; do
+    case "$param" in
+	mandos=off) exit 0;;
+    esac
+done
+
 chmod a=rwxt /tmp
 
 test -w /conf/conf.d/cryptroot

=== modified file 'mandos'
--- mandos	2009-01-06 05:10:34 +0000
+++ mandos	2009-01-18 10:56:33 +0000
@@ -66,7 +66,7 @@
 import ctypes
 import ctypes.util
 
-version = "1.0.3"
+version = "1.0.5"
 
 logger = logging.Logger('mandos')
 syslogger = (logging.handlers.SysLogHandler
@@ -178,7 +178,8 @@
 class Client(dbus.service.Object):
     """A representation of a client host served by this server.
     Attributes:
-    name:       string; from the config file, used in log messages
+    name:       string; from the config file, used in log messages and
+                        D-Bus identifiers
     fingerprint: string (40 or 32 hexadecimal digits); used to
                  uniquely identify the client
     secret:     bytestring; sent verbatim (over TLS) to client
@@ -503,6 +504,9 @@
                 dbus.String("checker_running"):
                     dbus.Boolean(self.checker is not None,
                                  variant_level=1),
+                dbus.String("object_path"):
+                    dbus.ObjectPath(self.dbus_object_path,
+                                    variant_level=1)
                 }, signature="sv")
     
     # IsStillValid - method
@@ -911,7 +915,8 @@
                       " files")
     parser.add_option("--no-dbus", action="store_false",
                       dest="use_dbus",
-                      help=optparse.SUPPRESS_HELP)
+                      help="Do not provide D-Bus system bus"
+                      " interface")
     options = parser.parse_args()[0]
     
     if options.check:
@@ -957,7 +962,6 @@
     # For convenience
     debug = server_settings["debug"]
     use_dbus = server_settings["use_dbus"]
-    use_dbus = False
     
     if not debug:
         syslogger.setLevel(logging.WARNING)
@@ -972,7 +976,7 @@
     # Parse config file with clients
     client_defaults = { "timeout": "1h",
                         "interval": "5m",
-                        "checker": "fping -q -- %(host)s",
+                        "checker": "fping -q -- %%(host)s",
                         "host": "",
                         }
     client_config = ConfigParser.SafeConfigParser(client_defaults)
@@ -993,23 +997,17 @@
     
     try:
         uid = pwd.getpwnam("_mandos").pw_uid
+        gid = pwd.getpwnam("_mandos").pw_gid
     except KeyError:
         try:
             uid = pwd.getpwnam("mandos").pw_uid
+            gid = pwd.getpwnam("mandos").pw_gid
         except KeyError:
             try:
                 uid = pwd.getpwnam("nobody").pw_uid
+                gid = pwd.getpwnam("nogroup").pw_gid
             except KeyError:
                 uid = 65534
-    try:
-        gid = pwd.getpwnam("_mandos").pw_gid
-    except KeyError:
-        try:
-            gid = pwd.getpwnam("mandos").pw_gid
-        except KeyError:
-            try:
-                gid = pwd.getpwnam("nogroup").pw_gid
-            except KeyError:
                 gid = 65534
     try:
         os.setuid(uid)
@@ -1101,28 +1099,28 @@
                 dbus.service.Object.__init__(self, bus,
                                              "/Mandos")
             _interface = u"org.mandos_system.Mandos"
-
+            
             @dbus.service.signal(_interface, signature="oa{sv}")
             def ClientAdded(self, objpath, properties):
                 "D-Bus signal"
                 pass
-
-            @dbus.service.signal(_interface, signature="o")
-            def ClientRemoved(self, objpath):
+            
+            @dbus.service.signal(_interface, signature="os")
+            def ClientRemoved(self, objpath, name):
                 "D-Bus signal"
                 pass
-
+            
             @dbus.service.method(_interface, out_signature="ao")
             def GetAllClients(self):
                 return dbus.Array(c.dbus_object_path for c in clients)
-
+            
             @dbus.service.method(_interface, out_signature="a{oa{sv}}")
             def GetAllClientsWithProperties(self):
                 return dbus.Dictionary(
                     ((c.dbus_object_path, c.GetAllProperties())
                      for c in clients),
                     signature="oa{sv}")
-
+            
             @dbus.service.method(_interface, in_signature="o")
             def RemoveClient(self, object_path):
                 for c in clients:
@@ -1132,15 +1130,29 @@
                         c.use_dbus = False
                         c.disable()
                         # Emit D-Bus signal
-                        self.ClientRemoved(object_path)
-                        return
-                raise KeyError
+                        self.ClientRemoved(object_path, c.name)
+                        return
+                raise KeyError
+            
+            @dbus.service.method(_interface, in_signature="s")
+            def RemoveClientByName(self, name):
+                for c in clients:
+                    if c.name == name:
+                        clients.remove(c)
+                        # Don't signal anything except ClientRemoved
+                        c.use_dbus = False
+                        c.disable()
+                        # Emit D-Bus signal
+                        self.ClientRemoved(c.dbus_object_path, name)
+                        return
+                raise KeyError
+            
             @dbus.service.method(_interface)
             def Quit(self):
                 main_loop.quit()
-
+            
             del _interface
-    
+        
         mandos_server = MandosServer()
     
     for client in clients:

=== modified file 'mandos-clients.conf.xml'
--- mandos-clients.conf.xml	2009-01-04 21:54:55 +0000
+++ mandos-clients.conf.xml	2009-01-08 03:54:06 +0000
@@ -3,7 +3,7 @@
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY CONFNAME "mandos-clients.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/clients.conf</filename>">
-<!ENTITY TIMESTAMP "2009-01-04">
+<!ENTITY TIMESTAMP "2009-01-08">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -170,7 +170,7 @@
 	    <varname>PATH</varname> will be searched.  The default
 	    value for the checker command is <quote><literal
 	    ><command>fping</command> <option>-q</option> <option
-	    >--</option> %(host)s</literal></quote>.
+	    >--</option> %%(host)s</literal></quote>.
 	  </para>
 	  <para>
 	    In addition to normal start time expansion, this option
@@ -351,7 +351,7 @@
 [DEFAULT]
 timeout = 1h
 interval = 5m
-checker = fping -q -- %(host)s
+checker = fping -q -- %%(host)s
 
 # Client "foo"
 [foo]

=== added file 'mandos-ctl'
--- mandos-ctl	1970-01-01 00:00:00 +0000
+++ mandos-ctl	2009-01-17 02:12:05 +0000
@@ -0,0 +1,187 @@
+#!/usr/bin/python
+# -*- mode: python; coding: utf-8 -*-
+
+from __future__ import division
+import sys
+import dbus
+from optparse import OptionParser
+import locale
+import datetime
+import re
+
+locale.setlocale(locale.LC_ALL, u'')
+
+tablewords = {
+    'name': u'Name',
+    'enabled': u'Enabled',
+    'timeout': u'Timeout',
+    'last_checked_ok': u'Last Successful Check',
+    'created': u'Created',
+    'interval': u'Interval',
+    'host': u'Host',
+    'fingerprint': u'Fingerprint',
+    'checker_running': u'Check Is Running',
+    'last_enabled': u'Last Enabled',
+    'checker': u'Checker',
+    }
+defaultkeywords = ('name', 'enabled', 'timeout', 'last_checked_ok',
+                   'checker')
+busname = 'org.mandos-system.Mandos'
+server_path = '/Mandos'
+server_interface = 'org.mandos_system.Mandos'
+client_interface = 'org.mandos_system.Mandos.Client'
+version = "1.0.5"
+
+bus = dbus.SystemBus()
+mandos_dbus_objc = bus.get_object(busname, server_path)
+mandos_serv = dbus.Interface(mandos_dbus_objc,
+                             dbus_interface = server_interface)
+mandos_clients = mandos_serv.GetAllClientsWithProperties()
+
+def datetime_to_milliseconds(dt):
+    "Return the 'timeout' attribute in milliseconds"
+    return ((dt.days * 24 * 60 * 60 * 1000)
+            + (dt.seconds * 1000)
+            + (dt.microseconds // 1000))
+
+def milliseconds_to_string(ms):
+    td = datetime.timedelta(0, 0, 0, ms)
+    return "%s%02d:%02d:%02d" % (("%dT" % td.days) if td.days else "", # days
+                           td.seconds // 3600,        # hours
+                           (td.seconds % 3600) // 60, # minutes
+                           (td.seconds % 60))         # seconds
+
+
+def string_to_delta(interval):
+    """Parse a string and return a datetime.timedelta
+
+    >>> string_to_delta('7d')
+    datetime.timedelta(7)
+    >>> string_to_delta('60s')
+    datetime.timedelta(0, 60)
+    >>> string_to_delta('60m')
+    datetime.timedelta(0, 3600)
+    >>> string_to_delta('24h')
+    datetime.timedelta(1)
+    >>> string_to_delta(u'1w')
+    datetime.timedelta(7)
+    >>> string_to_delta('5m 30s')
+    datetime.timedelta(0, 330)
+    """
+    timevalue = datetime.timedelta(0)
+    regexp = re.compile("\d+[dsmhw]")
+    
+    for s in regexp.findall(interval):
+        try:
+            suffix = unicode(s[-1])
+            value = int(s[:-1])
+            if suffix == u"d":
+                delta = datetime.timedelta(value)
+            elif suffix == u"s":
+                delta = datetime.timedelta(0, value)
+            elif suffix == u"m":
+                delta = datetime.timedelta(0, 0, 0, 0, value)
+            elif suffix == u"h":
+                delta = datetime.timedelta(0, 0, 0, 0, 0, value)
+            elif suffix == u"w":
+                delta = datetime.timedelta(0, 0, 0, 0, 0, 0, value)
+            else:
+                raise ValueError
+        except (ValueError, IndexError):
+            raise ValueError
+        timevalue += delta
+    return timevalue
+
+def print_clients(clients):
+    def valuetostring(value, keyword):
+        if type(value) is dbus.Boolean:
+            return u"Yes" if value else u"No"
+        if keyword in ("timeout", "interval"):
+            return milliseconds_to_string(value)
+        return unicode(value)
+    
+    format_string = u' '.join(u'%%-%ds' %
+                              max(len(tablewords[key]),
+                                  max(len(valuetostring(client[key], key))
+                                      for client in
+                                      clients))
+                              for key in keywords)
+    print format_string % tuple(tablewords[key] for key in keywords) 
+    for client in clients:
+        print format_string % tuple(valuetostring(client[key], key)
+                                    for key in keywords)
+
+parser = OptionParser(version = "%%prog %s" % version)
+parser.add_option("-a", "--all", action="store_true",
+                  help="Print all fields")
+parser.add_option("-e", "--enable", action="store_true",
+                  help="Enable specified client")
+parser.add_option("-d", "--disable", action="store_true",
+                  help="disable specified client")
+parser.add_option("-b", "--bump-timeout", action="store_true",
+                  help="Bump timeout of specified client")
+parser.add_option("--start-checker", action="store_true",
+                  help="Start checker for specified client")
+parser.add_option("--stop-checker", action="store_true",
+                  help="Stop checker for specified client")
+parser.add_option("-v", "--is-valid", action="store_true",
+                  help="Stop checker for specified client")
+parser.add_option("-c", "--checker", type="string",
+                  help="Set checker command for specified client")
+parser.add_option("-t", "--timeout", type="string",
+                  help="Set timeout for specified client")
+parser.add_option("-i", "--interval", type="string",
+                  help="Set checker interval for specified client")
+parser.add_option("-H", "--host", type="string",
+                  help="Set host for specified client")
+parser.add_option("-s", "--secret", type="string",
+                  help="Set password blob (file) for specified client")
+options, client_names = parser.parse_args()
+
+clients=[]
+for name in client_names:
+    for path, client in mandos_clients.iteritems():
+        if client['name'] == name:
+            client_objc = bus.get_object(busname, path)
+            clients.append(dbus.Interface(client_objc,
+                                          dbus_interface
+                                          = client_interface))
+            break
+    else:
+        print >> sys.stderr, "Client not found on server: %r" % name
+        sys.exit(1)
+
+if not clients:
+    keywords = defaultkeywords
+    if options.all:
+        keywords = ('name', 'enabled', 'timeout', 'last_checked_ok',
+                    'created', 'interval', 'host', 'fingerprint',
+                    'checker_running', 'last_enabled', 'checker')
+    print_clients(mandos_clients.values())
+    
+for client in clients:
+    if options.enable:
+        client.Enable()
+    if options.disable:
+        client.Disable()
+    if options.bump_timeout:
+        client.BumpTimeout()
+    if options.start_checker:
+        client.StartChecker()
+    if options.stop_checker:
+        client.StopChecker()
+    if options.is_valid:
+        sys.exit(0 if client.IsStillValid() else 1)
+    if options.checker:
+        client.SetChecker(options.checker)
+    if options.host:
+        client.SetHost(options.host)
+    if options.interval:
+        client.SetInterval(datetime_to_milliseconds
+                           (string_to_delta(options.interval)))
+    if options.timeout:
+        client.SetTimeout(datetime_to_milliseconds
+                          (string_to_delta(options.timeout)))
+    if options.secret:
+        client.SetSecret(dbus.ByteArray(open(options.secret, 'rb').read()))
+    

=== modified file 'mandos-keygen'
--- mandos-keygen	2009-01-06 05:10:34 +0000
+++ mandos-keygen	2009-01-17 09:46:54 +0000
@@ -21,7 +21,7 @@
 # Contact the authors at <mandos@fukt.bsnet.se>.
 # 
 
-VERSION="1.0.3"
+VERSION="1.0.5"
 
 KEYDIR="/etc/keys/mandos"
 KEYTYPE=DSA
@@ -147,7 +147,7 @@
 	echo "Invalid key length" >&2
 	exit 1
     fi
-
+    
     if [ -z "$KEYEXPIRE" ]; then
 	echo "Empty key expiration" >&2
 	exit 1
@@ -172,7 +172,7 @@
     if [ -n "$KEYEMAIL" ]; then
 	KEYEMAILLINE="Name-Email: $KEYEMAIL"
     fi
-
+    
     # Create temporary gpg batch file
     BATCHFILE="`mktemp -t mandos-keygen-batch.XXXXXXXXXX`"
 fi
@@ -284,7 +284,7 @@
 	stty echo
 	if [ "$first" != "$second" ]; then
 	    echo -e "Passphrase mismatch" >&2
-	    false
+	    touch "$RINGDIR"/mismatch
 	else
 	    echo -n "$first"
 	fi
@@ -292,9 +292,9 @@
 	--homedir "$RINGDIR" --trust-model always --armor --encrypt \
 	--sign --recipient "$FINGERPRINT" --comment "$FILECOMMENT" \
 	> "$SECFILE"
-    status="${PIPESTATUS[0]}"
-    if [ "$status" -ne 0 ]; then
-	exit "$status"
+    if [ -e "$RINGDIR"/mismatch ]; then
+	rm --force "$RINGDIR"/mismatch
+	exit 1
     fi
     
     cat <<-EOF

=== modified file 'mandos.lsm'
--- mandos.lsm	2009-01-06 05:10:34 +0000
+++ mandos.lsm	2009-01-17 02:12:05 +0000
@@ -1,7 +1,7 @@
 Begin4
 Title:		Mandos
-Version:	1.0.3
-Entered-date:	2009-01-06
+Version:	1.0.5
+Entered-date:	2009-01-17
 Description:	The Mandos system allows computers to have encrypted
 root file systems and at the same time be capable of remote and/or
 unattended reboots.
@@ -12,9 +12,9 @@
 Maintained-by:	teddy@fukt.bsnet.se (Teddy Hogeborn),
 		belorn@fukt.bsnet.se (Björn Påhlsson)
 Primary-site: http://www.fukt.bsnet.se/mandos
-	      92K mandos_1.0.3.orig.tar.gz
+	      94K mandos_1.0.5.orig.tar.gz
 Alternate-site: ftp://ftp.fukt.bsnet.se/pub/mandos
-		92K mandos_1.0.3.orig.tar.gz
+		94K mandos_1.0.5.orig.tar.gz
 Platforms:	Requires GCC, GNU libC, Avahi, GnuPG, Python 2.4, and
 various other libraries.  While made for Debian GNU/Linux, it is
 probably portable to other distributions, but not other Unixes.

=== modified file 'mandos.xml'
--- mandos.xml	2009-01-05 23:26:06 +0000
+++ mandos.xml	2009-01-15 05:27:55 +0000
@@ -85,6 +85,8 @@
       <replaceable>DIRECTORY</replaceable></option></arg>
       <sbr/>
       <arg><option>--debug</option></arg>
+      <sbr/>
+      <arg><option>--no-dbus</option></arg>
     </cmdsynopsis>
     <cmdsynopsis>
       <command>&COMMANDNAME;</command>
@@ -229,6 +231,16 @@
 	  </para>
 	</listitem>
       </varlistentry>
+      
+      <varlistentry>
+	<term><option>--no-dbus</option></term>
+	<listitem>
+	  <xi:include href="mandos-options.xml" xpointer="dbus"/>
+	  <para>
+	    See also <xref linkend="dbus_interface"/>.
+	  </para>
+	</listitem>
+      </varlistentry>
     </variablelist>
   </refsect1>
   
@@ -324,6 +336,16 @@
     </para>
   </refsect1>
   
+  <refsect1 id="dbus_interface">
+    <title>D-BUS INTERFACE</title>
+    <para>
+      The server will by default provide a D-Bus system bus interface.
+      This interface will only be accessible by the root user or a
+      Mandos-specific user, if such a user exists.
+      <!-- XXX -->
+    </para>
+  </refsect1>
+
   <refsect1 id="exit_status">
     <title>EXIT STATUS</title>
     <para>

=== modified file 'plugin-runner.c'
--- plugin-runner.c	2009-01-04 21:54:55 +0000
+++ plugin-runner.c	2009-01-17 02:34:57 +0000
@@ -64,6 +64,7 @@
 				   sigprocmask(), SIG_BLOCK, SIGCHLD,
 				   SIG_UNBLOCK, kill() */
 #include <errno.h>		/* errno, EBADF */
+#include <inttypes.h>		/* intmax_t, SCNdMAX, PRIdMAX,  */
 
 #define BUFFER_SIZE 256
 
@@ -99,21 +100,22 @@
    or if none is found, creates a new one */
 static plugin *getplugin(char *name){
   /* Check for exiting plugin with that name */
-  for (plugin *p = plugin_list; p != NULL; p = p->next){
-    if ((p->name == name)
-	or (p->name and name and (strcmp(p->name, name) == 0))){
+  for(plugin *p = plugin_list; p != NULL; p = p->next){
+    if((p->name == name)
+       or (p->name and name and (strcmp(p->name, name) == 0))){
       return p;
     }
   }
   /* Create a new plugin */
   plugin *new_plugin = malloc(sizeof(plugin));
-  if (new_plugin == NULL){
+  if(new_plugin == NULL){
     return NULL;
   }
   char *copy_name = NULL;
   if(name != NULL){
     copy_name = strdup(name);
     if(copy_name == NULL){
+      free(new_plugin);
       return NULL;
     }
   }
@@ -124,7 +126,7 @@
 			   .next = plugin_list };
   
   new_plugin->argv = malloc(sizeof(char *) * 2);
-  if (new_plugin->argv == NULL){
+  if(new_plugin->argv == NULL){
     free(copy_name);
     free(new_plugin);
     return NULL;
@@ -230,7 +232,7 @@
       break;
     }
     if(pid == -1){
-      if (errno != ECHILD){
+      if(errno != ECHILD){
 	perror("waitpid");
       }
       /* No child processes */
@@ -308,7 +310,9 @@
   struct dirent *dirst;
   struct stat st;
   fd_set rfds_all;
-  int ret, maxfd = 0;
+  int ret, numchars, maxfd = 0;
+  ssize_t sret;
+  intmax_t tmpmax;
   uid_t uid = 65534;
   gid_t gid = 65534;
   bool debug = false;
@@ -371,11 +375,11 @@
     { .name = NULL }
   };
   
-  error_t parse_opt (int key, char *arg, __attribute__((unused))
-		     struct argp_state *state) {
-    switch (key) {
+  error_t parse_opt(int key, char *arg, __attribute__((unused))
+		    struct argp_state *state) {
+    switch(key) {
     case 'g': 			/* --global-options */
-      if (arg != NULL){
+      if(arg != NULL){
 	char *p;
 	while((p = strsep(&arg, ",")) != NULL){
 	  if(p[0] == '\0'){
@@ -397,7 +401,7 @@
       }
       break;
     case 'o':			/* --options-for */
-      if (arg != NULL){
+      if(arg != NULL){
 	char *p_name = strsep(&arg, ":");
 	if(p_name[0] == '\0' or arg == NULL){
 	  break;
@@ -434,7 +438,7 @@
       }
       break;
     case 'd':			/* --disable */
-      if (arg != NULL){
+      if(arg != NULL){
 	plugin *p = getplugin(arg);
 	if(p == NULL){
 	  return ARGP_ERR_UNKNOWN;
@@ -443,7 +447,7 @@
       }
       break;
     case 'e':			/* --enable */
-      if (arg != NULL){
+      if(arg != NULL){
 	plugin *p = getplugin(arg);
 	if(p == NULL){
 	  return ARGP_ERR_UNKNOWN;
@@ -462,14 +466,32 @@
       /* This is already done by parse_opt_config_file() */
       break;
     case 130:			/* --userid */
-      uid = (uid_t)strtol(arg, NULL, 10);
+      ret = sscanf(arg, "%" SCNdMAX "%n", &tmpmax, &numchars);
+      if(ret < 1 or tmpmax != (uid_t)tmpmax
+	 or arg[numchars] != '\0'){
+	fprintf(stderr, "Bad user ID number: \"%s\", using %"
+		PRIdMAX "\n", arg, (intmax_t)uid);
+      } else {
+	uid = (uid_t)tmpmax;
+      }
       break;
     case 131:			/* --groupid */
-      gid = (gid_t)strtol(arg, NULL, 10);
+      ret = sscanf(arg, "%" SCNdMAX "%n", &tmpmax, &numchars);
+      if(ret < 1 or tmpmax != (gid_t)tmpmax
+	 or arg[numchars] != '\0'){
+	fprintf(stderr, "Bad group ID number: \"%s\", using %"
+		PRIdMAX "\n", arg, (intmax_t)gid);
+      } else {
+	gid = (gid_t)tmpmax;
+      }
       break;
     case 132:			/* --debug */
       debug = true;
       break;
+/*
+ * When adding more options before this line, remember to also add a
+ * "case" to the "parse_opt_config_file" function below.
+ */
     case ARGP_KEY_ARG:
       /* Cryptsetup always passes an argument, which is an empty
 	 string if "none" was specified in /etc/crypttab.  So if
@@ -488,10 +510,10 @@
   
   /* This option parser is the same as parse_opt() above, except it
      ignores everything but the --config-file option. */
-  error_t parse_opt_config_file (int key, char *arg,
-				 __attribute__((unused))
-				 struct argp_state *state) {
-    switch (key) {
+  error_t parse_opt_config_file(int key, char *arg,
+				__attribute__((unused))
+				struct argp_state *state) {
+    switch(key) {
     case 'g': 			/* --global-options */
     case 'G':			/* --global-env */
     case 'o':			/* --options-for */
@@ -524,10 +546,10 @@
 		       .args_doc = "",
 		       .doc = "Mandos plugin runner -- Run plugins" };
   
-  /* Parse using the parse_opt_config_file in order to get the custom
+  /* Parse using parse_opt_config_file() in order to get the custom
      config file location, if any. */
-  ret = argp_parse (&argp, argc, argv, ARGP_IN_ORDER, 0, NULL);
-  if (ret == ARGP_ERR_UNKNOWN){
+  ret = argp_parse(&argp, argc, argv, ARGP_IN_ORDER, 0, NULL);
+  if(ret == ARGP_ERR_UNKNOWN){
     fprintf(stderr, "Unknown error while parsing arguments\n");
     exitstatus = EXIT_FAILURE;
     goto fallback;
@@ -537,7 +559,7 @@
   argp.parser = parse_opt;
   
   /* Open the configfile if available */
-  if (argfile == NULL){
+  if(argfile == NULL){
     conffp = fopen(AFILE, "r");
   } else {
     conffp = fopen(argfile, "r");
@@ -546,7 +568,6 @@
     char *org_line = NULL;
     char *p, *arg, *new_arg, *line;
     size_t size = 0;
-    ssize_t sret;
     const char whitespace_delims[] = " \r\t\f\v\n";
     const char comment_delim[] = "#";
 
@@ -599,7 +620,7 @@
   } else {
     /* Check for harmful errors and go to fallback. Other errors might
        not affect opening plugins */
-    if (errno == EMFILE or errno == ENFILE or errno == ENOMEM){
+    if(errno == EMFILE or errno == ENFILE or errno == ENOMEM){
       perror("fopen");
       exitstatus = EXIT_FAILURE;
       goto fallback;
@@ -608,9 +629,9 @@
   /* If there was any arguments from configuration file,
      pass them to parser as command arguments */
   if(custom_argv != NULL){
-    ret = argp_parse (&argp, custom_argc, custom_argv, ARGP_IN_ORDER,
-		      0, NULL);
-    if (ret == ARGP_ERR_UNKNOWN){
+    ret = argp_parse(&argp, custom_argc, custom_argv, ARGP_IN_ORDER,
+		     0, NULL);
+    if(ret == ARGP_ERR_UNKNOWN){
       fprintf(stderr, "Unknown error while parsing arguments\n");
       exitstatus = EXIT_FAILURE;
       goto fallback;
@@ -619,8 +640,8 @@
   
   /* Parse actual command line arguments, to let them override the
      config file */
-  ret = argp_parse (&argp, argc, argv, ARGP_IN_ORDER, 0, NULL);
-  if (ret == ARGP_ERR_UNKNOWN){
+  ret = argp_parse(&argp, argc, argv, ARGP_IN_ORDER, 0, NULL);
+  if(ret == ARGP_ERR_UNKNOWN){
     fprintf(stderr, "Unknown error while parsing arguments\n");
     exitstatus = EXIT_FAILURE;
     goto fallback;
@@ -633,7 +654,7 @@
       for(char **a = p->argv; *a != NULL; a++){
 	fprintf(stderr, "\tArg: %s\n", *a);
       }
-      fprintf(stderr, "...and %u environment variables\n", p->envc);
+      fprintf(stderr, "...and %d environment variables\n", p->envc);
       for(char **a = p->environ; *a != NULL; a++){
 	fprintf(stderr, "\t%s\n", *a);
       }
@@ -642,15 +663,15 @@
   
   /* Strip permissions down to nobody */
   ret = setuid(uid);
-  if (ret == -1){
+  if(ret == -1){
     perror("setuid");
   }  
   setgid(gid);
-  if (ret == -1){
+  if(ret == -1){
     perror("setgid");
   }
   
-  if (plugindir == NULL){
+  if(plugindir == NULL){
     dir = opendir(PDIR);
   } else {
     dir = opendir(plugindir);
@@ -683,7 +704,7 @@
     
     /* All directory entries have been processed */
     if(dirst == NULL){
-      if (errno == EBADF){
+      if(errno == EBADF){
 	perror("readdir");
 	exitstatus = EXIT_FAILURE;
 	goto fallback;
@@ -749,14 +770,14 @@
     }
     
     ret = stat(filename, &st);
-    if (ret == -1){
+    if(ret == -1){
       perror("stat");
       free(filename);
       continue;
     }
 
     /* Ignore non-executable files */
-    if (not S_ISREG(st.st_mode)	or (access(filename, X_OK) != 0)){
+    if(not S_ISREG(st.st_mode) or (access(filename, X_OK) != 0)){
       if(debug){
 	fprintf(stderr, "Ignoring plugin dir entry \"%s\""
 		" with bad type or mode\n", filename);
@@ -809,7 +830,7 @@
     
     int pipefd[2];
     ret = pipe(pipefd);
-    if (ret == -1){
+    if(ret == -1){
       perror("pipe");
       exitstatus = EXIT_FAILURE;
       goto fallback;
@@ -828,7 +849,7 @@
       goto fallback;
     }
     /* Block SIGCHLD until process is safely in process list */
-    ret = sigprocmask (SIG_BLOCK, &sigchld_action.sa_mask, NULL);
+    ret = sigprocmask(SIG_BLOCK, &sigchld_action.sa_mask, NULL);
     if(ret < 0){
       perror("sigprocmask");
       exitstatus = EXIT_FAILURE;
@@ -882,9 +903,9 @@
     close(pipefd[1]);		/* Close unused write end of pipe */
     free(filename);
     plugin *new_plugin = getplugin(dirst->d_name);
-    if (new_plugin == NULL){
+    if(new_plugin == NULL){
       perror("getplugin");
-      ret = sigprocmask (SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
+      ret = sigprocmask(SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
       if(ret < 0){
         perror("sigprocmask");
       }
@@ -897,7 +918,7 @@
     
     /* Unblock SIGCHLD so signal handler can be run if this process
        has already completed */
-    ret = sigprocmask (SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
+    ret = sigprocmask(SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
     if(ret < 0){
       perror("sigprocmask");
       exitstatus = EXIT_FAILURE;
@@ -906,7 +927,7 @@
     
     FD_SET(new_plugin->fd, &rfds_all);
     
-    if (maxfd < new_plugin->fd){
+    if(maxfd < new_plugin->fd){
       maxfd = new_plugin->fd;
     }
   }
@@ -929,7 +950,7 @@
   while(plugin_list){
     fd_set rfds = rfds_all;
     int select_ret = select(maxfd+1, &rfds, NULL, NULL, NULL);
-    if (select_ret == -1){
+    if(select_ret == -1){
       perror("select");
       exitstatus = EXIT_FAILURE;
       goto fallback;
@@ -946,16 +967,16 @@
 
 	  if(debug){
 	    if(WIFEXITED(proc->status)){
-	      fprintf(stderr, "Plugin %u exited with status %d\n",
-		      (unsigned int) (proc->pid),
+	      fprintf(stderr, "Plugin %" PRIdMAX " exited with status"
+		      " %d\n", (intmax_t) (proc->pid),
 		      WEXITSTATUS(proc->status));
 	    } else if(WIFSIGNALED(proc->status)) {
-	      fprintf(stderr, "Plugin %u killed by signal %d\n",
-		      (unsigned int) (proc->pid),
+	      fprintf(stderr, "Plugin %" PRIdMAX " killed by signal"
+		      " %d\n", (intmax_t) (proc->pid),
 		      WTERMSIG(proc->status));
 	    } else if(WCOREDUMP(proc->status)){
-	      fprintf(stderr, "Plugin %d dumped core\n",
-		      (unsigned int) (proc->pid));
+	      fprintf(stderr, "Plugin %" PRIdMAX " dumped core\n",
+		      (intmax_t) (proc->pid));
 	    }
 	  }
 	  
@@ -975,8 +996,8 @@
 	  proc = next_plugin;
 	  
 	  /* We are done modifying process list, so unblock signal */
-	  ret = sigprocmask (SIG_UNBLOCK, &sigchld_action.sa_mask,
-			     NULL);
+	  ret = sigprocmask(SIG_UNBLOCK, &sigchld_action.sa_mask,
+			    NULL);
 	  if(ret < 0){
 	    perror("sigprocmask");
 	    exitstatus = EXIT_FAILURE;
@@ -1011,7 +1032,7 @@
       if(proc->buffer_length + BUFFER_SIZE > proc->buffer_size){
 	proc->buffer = realloc(proc->buffer, proc->buffer_size
 			       + (size_t) BUFFER_SIZE);
-	if (proc->buffer == NULL){
+	if(proc->buffer == NULL){
 	  perror("malloc");
 	  exitstatus = EXIT_FAILURE;
 	  goto fallback;
@@ -1019,18 +1040,18 @@
 	proc->buffer_size += BUFFER_SIZE;
       }
       /* Read from the process */
-      ret = read(proc->fd, proc->buffer + proc->buffer_length,
-		 BUFFER_SIZE);
-      if(ret < 0){
+      sret = read(proc->fd, proc->buffer + proc->buffer_length,
+		  BUFFER_SIZE);
+      if(sret < 0){
 	/* Read error from this process; ignore the error */
 	proc = proc->next;
 	continue;
       }
-      if(ret == 0){
+      if(sret == 0){
 	/* got EOF */
 	proc->eof = true;
       } else {
-	proc->buffer_length += (size_t) ret;
+	proc->buffer_length += (size_t) sret;
       }
     }
   }

=== modified file 'plugin-runner.xml'
--- plugin-runner.xml	2009-01-04 21:54:55 +0000
+++ plugin-runner.xml	2009-01-18 06:41:57 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "plugin-runner">
-<!ENTITY TIMESTAMP "2009-01-04">
+<!ENTITY TIMESTAMP "2009-01-17">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -579,7 +579,7 @@
       <para>
 
 <!-- do not wrap this line -->
-<userinput>&COMMANDNAME;  --config-file=/etc/mandos/plugin-runner.conf --plugin-dir /usr/lib/mandos/plugins.d --options-for=mandos-client:--pubkey=/etc/keys/mandos/pubkey.txt,--seckey=/etc/keys/mandos/seckey.txt</userinput>
+<userinput>cd /etc/keys/mandos; &COMMANDNAME;  --config-file=/etc/mandos/plugin-runner.conf --plugin-dir /usr/lib/mandos/plugins.d --options-for=mandos-client:--pubkey=pubkey.txt,--seckey=seckey.txt</userinput>
 
       </para>
     </informalexample>

=== modified file 'plugins.d/askpass-fifo.c'
--- plugins.d/askpass-fifo.c	2009-01-04 21:54:55 +0000
+++ plugins.d/askpass-fifo.c	2009-01-10 06:00:50 +0000
@@ -1,6 +1,6 @@
 /*  -*- coding: utf-8 -*- */
 /*
- * Passprompt - Read a password from a FIFO and output it
+ * Askpass-FIFO - Read a password from a FIFO and output it
  * 
  * Copyright © 2008,2009 Teddy Hogeborn
  * Copyright © 2008,2009 Björn Påhlsson
@@ -43,14 +43,14 @@
   
   /* Create FIFO */
   const char passfifo[] = "/lib/cryptsetup/passfifo";
-  ret = TEMP_FAILURE_RETRY(mkfifo(passfifo, S_IRUSR | S_IWUSR));
+  ret = (int)TEMP_FAILURE_RETRY(mkfifo(passfifo, S_IRUSR | S_IWUSR));
   if(ret == -1 and errno != EEXIST){
     perror("mkfifo");
     return EXIT_FAILURE;
   }
   
   /* Open FIFO */
-  int fifo_fd = TEMP_FAILURE_RETRY(open(passfifo, O_RDONLY));
+  int fifo_fd = (int)TEMP_FAILURE_RETRY(open(passfifo, O_RDONLY));
   if(fifo_fd == -1){
     perror("open");
     return EXIT_FAILURE;

=== modified file 'plugins.d/mandos-client.c'
--- plugins.d/mandos-client.c	2009-01-04 21:54:55 +0000
+++ plugins.d/mandos-client.c	2009-01-14 14:20:17 +0000
@@ -1,6 +1,6 @@
 /*  -*- coding: utf-8 -*- */
 /*
- * Mandos client - get and decrypt data from a Mandos server
+ * Mandos-client - get and decrypt data from a Mandos server
  *
  * This program is partly derived from an example program for an Avahi
  * service browser, downloaded from
@@ -36,7 +36,7 @@
 #define _GNU_SOURCE		/* TEMP_FAILURE_RETRY(), asprintf() */
 
 #include <stdio.h>		/* fprintf(), stderr, fwrite(),
-				   stdout, ferror() */
+				   stdout, ferror(), sscanf */
 #include <stdint.h> 		/* uint16_t, uint32_t */
 #include <stddef.h>		/* NULL, size_t, ssize_t */
 #include <stdlib.h> 		/* free(), EXIT_SUCCESS, EXIT_FAILURE,
@@ -48,14 +48,16 @@
 #include <sys/types.h>		/* socket(), inet_pton(), sockaddr,
 				   sockaddr_in6, PF_INET6,
 				   SOCK_STREAM, INET6_ADDRSTRLEN,
-				   uid_t, gid_t, open(), opendir(), DIR */
+				   uid_t, gid_t, open(), opendir(),
+				   DIR */
 #include <sys/stat.h>		/* open() */
 #include <sys/socket.h>		/* socket(), struct sockaddr_in6,
 				   struct in6_addr, inet_pton(),
 				   connect() */
 #include <fcntl.h>		/* open() */
-#include <dirent.h>		/* opendir(), struct dirent, readdir() */
-#include <inttypes.h>		/* PRIu16 */
+#include <dirent.h>		/* opendir(), struct dirent, readdir()
+				 */
+#include <inttypes.h>		/* PRIu16, intmax_t, SCNdMAX */
 #include <assert.h>		/* assert() */
 #include <errno.h>		/* perror(), errno */
 #include <time.h>		/* time() */
@@ -67,7 +69,7 @@
 				   getuid(), getgid(), setuid(),
 				   setgid() */
 #include <arpa/inet.h>		/* inet_pton(), htons */
-#include <iso646.h>		/* not, and */
+#include <iso646.h>		/* not, and, or */
 #include <argp.h>		/* struct argp_option, error_t, struct
 				   argp_state, struct argp,
 				   argp_parse(), ARGP_KEY_ARG,
@@ -90,7 +92,8 @@
 				   gnutls_*
 				   init_gnutls_session(),
 				   GNUTLS_* */
-#include <gnutls/openpgp.h>     /* gnutls_certificate_set_openpgp_key_file(),
+#include <gnutls/openpgp.h>
+			  /* gnutls_certificate_set_openpgp_key_file(),
 				   GNUTLS_OPENPGP_FMT_BASE64 */
 
 /* GPGME */
@@ -129,9 +132,9 @@
  */
 size_t adjustbuffer(char **buffer, size_t buffer_length,
 		  size_t buffer_capacity){
-  if (buffer_length + BUFFER_SIZE > buffer_capacity){
+  if(buffer_length + BUFFER_SIZE > buffer_capacity){
     *buffer = realloc(*buffer, buffer_capacity + BUFFER_SIZE);
-    if (buffer == NULL){
+    if(buffer == NULL){
       return 0;
     }
     buffer_capacity += BUFFER_SIZE;
@@ -156,27 +159,27 @@
     int fd;
     gpgme_data_t pgp_data;
     
-    fd = TEMP_FAILURE_RETRY(open(filename, O_RDONLY));
+    fd = (int)TEMP_FAILURE_RETRY(open(filename, O_RDONLY));
     if(fd == -1){
       perror("open");
       return false;
     }
     
     rc = gpgme_data_new_from_fd(&pgp_data, fd);
-    if (rc != GPG_ERR_NO_ERROR){
+    if(rc != GPG_ERR_NO_ERROR){
       fprintf(stderr, "bad gpgme_data_new_from_fd: %s: %s\n",
 	      gpgme_strsource(rc), gpgme_strerror(rc));
       return false;
     }
     
     rc = gpgme_op_import(mc->ctx, pgp_data);
-    if (rc != GPG_ERR_NO_ERROR){
+    if(rc != GPG_ERR_NO_ERROR){
       fprintf(stderr, "bad gpgme_op_import: %s: %s\n",
 	      gpgme_strsource(rc), gpgme_strerror(rc));
       return false;
     }
     
-    ret = TEMP_FAILURE_RETRY(close(fd));
+    ret = (int)TEMP_FAILURE_RETRY(close(fd));
     if(ret == -1){
       perror("close");
     }
@@ -184,22 +187,22 @@
     return true;
   }
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "Initialize gpgme\n");
   }
   
   /* Init GPGME */
   gpgme_check_version(NULL);
   rc = gpgme_engine_check_version(GPGME_PROTOCOL_OpenPGP);
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_engine_check_version: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     return false;
   }
   
     /* Set GPGME home directory for the OpenPGP engine only */
-  rc = gpgme_get_engine_info (&engine_info);
-  if (rc != GPG_ERR_NO_ERROR){
+  rc = gpgme_get_engine_info(&engine_info);
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_get_engine_info: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     return false;
@@ -219,13 +222,13 @@
   
   /* Create new GPGME "context" */
   rc = gpgme_new(&(mc->ctx));
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_new: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     return false;
   }
   
-  if (not import_key(pubkey) or not import_key(seckey)){
+  if(not import_key(pubkey) or not import_key(seckey)){
     return false;
   }
   
@@ -236,24 +239,24 @@
  * Decrypt OpenPGP data.
  * Returns -1 on error
  */
-static ssize_t pgp_packet_decrypt (const mandos_context *mc,
-				   const char *cryptotext,
-				   size_t crypto_size,
-				   char **plaintext){
+static ssize_t pgp_packet_decrypt(const mandos_context *mc,
+				  const char *cryptotext,
+				  size_t crypto_size,
+				  char **plaintext){
   gpgme_data_t dh_crypto, dh_plain;
   gpgme_error_t rc;
   ssize_t ret;
   size_t plaintext_capacity = 0;
   ssize_t plaintext_length = 0;
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "Trying to decrypt OpenPGP data\n");
   }
   
   /* Create new GPGME data buffer from memory cryptotext */
   rc = gpgme_data_new_from_mem(&dh_crypto, cryptotext, crypto_size,
 			       0);
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_data_new_from_mem: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     return -1;
@@ -261,7 +264,7 @@
   
   /* Create new empty GPGME data buffer for the plaintext */
   rc = gpgme_data_new(&dh_plain);
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_data_new: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     gpgme_data_release(dh_crypto);
@@ -271,14 +274,14 @@
   /* Decrypt data from the cryptotext data buffer to the plaintext
      data buffer */
   rc = gpgme_op_decrypt(mc->ctx, dh_crypto, dh_plain);
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_op_decrypt: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     plaintext_length = -1;
-    if (debug){
+    if(debug){
       gpgme_decrypt_result_t result;
       result = gpgme_op_decrypt_result(mc->ctx);
-      if (result == NULL){
+      if(result == NULL){
 	fprintf(stderr, "gpgme_op_decrypt_result failed\n");
       } else {
 	fprintf(stderr, "Unsupported algorithm: %s\n",
@@ -311,7 +314,7 @@
   }
   
   /* Seek back to the beginning of the GPGME plaintext data buffer */
-  if (gpgme_data_seek(dh_plain, (off_t) 0, SEEK_SET) == -1){
+  if(gpgme_data_seek(dh_plain, (off_t)0, SEEK_SET) == -1){
     perror("gpgme_data_seek");
     plaintext_length = -1;
     goto decrypt_end;
@@ -322,7 +325,7 @@
     plaintext_capacity = adjustbuffer(plaintext,
 				      (size_t)plaintext_length,
 				      plaintext_capacity);
-    if (plaintext_capacity == 0){
+    if(plaintext_capacity == 0){
 	perror("adjustbuffer");
 	plaintext_length = -1;
 	goto decrypt_end;
@@ -331,7 +334,7 @@
     ret = gpgme_data_read(dh_plain, *plaintext + plaintext_length,
 			  BUFFER_SIZE);
     /* Print the data, if any */
-    if (ret == 0){
+    if(ret == 0){
       /* EOF */
       break;
     }
@@ -361,9 +364,10 @@
   return plaintext_length;
 }
 
-static const char * safer_gnutls_strerror (int value) {
-  const char *ret = gnutls_strerror (value); /* Spurious warning */
-  if (ret == NULL)
+static const char * safer_gnutls_strerror(int value) {
+  const char *ret = gnutls_strerror(value); /* Spurious warning from
+					       -Wunreachable-code */
+  if(ret == NULL)
     ret = "(unknown)";
   return ret;
 }
@@ -384,13 +388,13 @@
   }
   
   ret = gnutls_global_init();
-  if (ret != GNUTLS_E_SUCCESS) {
-    fprintf (stderr, "GnuTLS global_init: %s\n",
-	     safer_gnutls_strerror(ret));
+  if(ret != GNUTLS_E_SUCCESS) {
+    fprintf(stderr, "GnuTLS global_init: %s\n",
+	    safer_gnutls_strerror(ret));
     return -1;
   }
   
-  if (debug){
+  if(debug){
     /* "Use a log level over 10 to enable all debugging options."
      * - GnuTLS manual
      */
@@ -400,11 +404,13 @@
   
   /* OpenPGP credentials */
   gnutls_certificate_allocate_credentials(&mc->cred);
-  if (ret != GNUTLS_E_SUCCESS){
-    fprintf (stderr, "GnuTLS memory error: %s\n", /* Spurious
-						     warning */
-	     safer_gnutls_strerror(ret));
-    gnutls_global_deinit ();
+  if(ret != GNUTLS_E_SUCCESS){
+    fprintf(stderr, "GnuTLS memory error: %s\n", /* Spurious warning
+						  * from
+						  * -Wunreachable-code
+						  */
+	    safer_gnutls_strerror(ret));
+    gnutls_global_deinit();
     return -1;
   }
   
@@ -417,7 +423,7 @@
   ret = gnutls_certificate_set_openpgp_key_file
     (mc->cred, pubkeyfilename, seckeyfilename,
      GNUTLS_OPENPGP_FMT_BASE64);
-  if (ret != GNUTLS_E_SUCCESS) {
+  if(ret != GNUTLS_E_SUCCESS) {
     fprintf(stderr,
 	    "Error[%d] while reading the OpenPGP key pair ('%s',"
 	    " '%s')\n", ret, pubkeyfilename, seckeyfilename);
@@ -428,15 +434,15 @@
   
   /* GnuTLS server initialization */
   ret = gnutls_dh_params_init(&mc->dh_params);
-  if (ret != GNUTLS_E_SUCCESS) {
-    fprintf (stderr, "Error in GnuTLS DH parameter initialization:"
-	     " %s\n", safer_gnutls_strerror(ret));
+  if(ret != GNUTLS_E_SUCCESS) {
+    fprintf(stderr, "Error in GnuTLS DH parameter initialization:"
+	    " %s\n", safer_gnutls_strerror(ret));
     goto globalfail;
   }
   ret = gnutls_dh_params_generate2(mc->dh_params, mc->dh_bits);
-  if (ret != GNUTLS_E_SUCCESS) {
-    fprintf (stderr, "Error in GnuTLS prime generation: %s\n",
-	     safer_gnutls_strerror(ret));
+  if(ret != GNUTLS_E_SUCCESS) {
+    fprintf(stderr, "Error in GnuTLS prime generation: %s\n",
+	    safer_gnutls_strerror(ret));
     goto globalfail;
   }
   
@@ -457,7 +463,7 @@
   int ret;
   /* GnuTLS session creation */
   ret = gnutls_init(session, GNUTLS_SERVER);
-  if (ret != GNUTLS_E_SUCCESS){
+  if(ret != GNUTLS_E_SUCCESS){
     fprintf(stderr, "Error in GnuTLS session initialization: %s\n",
 	    safer_gnutls_strerror(ret));
   }
@@ -465,29 +471,29 @@
   {
     const char *err;
     ret = gnutls_priority_set_direct(*session, mc->priority, &err);
-    if (ret != GNUTLS_E_SUCCESS) {
+    if(ret != GNUTLS_E_SUCCESS) {
       fprintf(stderr, "Syntax error at: %s\n", err);
       fprintf(stderr, "GnuTLS error: %s\n",
 	      safer_gnutls_strerror(ret));
-      gnutls_deinit (*session);
+      gnutls_deinit(*session);
       return -1;
     }
   }
   
   ret = gnutls_credentials_set(*session, GNUTLS_CRD_CERTIFICATE,
 			       mc->cred);
-  if (ret != GNUTLS_E_SUCCESS) {
+  if(ret != GNUTLS_E_SUCCESS) {
     fprintf(stderr, "Error setting GnuTLS credentials: %s\n",
 	    safer_gnutls_strerror(ret));
-    gnutls_deinit (*session);
+    gnutls_deinit(*session);
     return -1;
   }
   
   /* ignore client certificate if any. */
-  gnutls_certificate_server_set_request (*session,
-					 GNUTLS_CERT_IGNORE);
+  gnutls_certificate_server_set_request(*session,
+					GNUTLS_CERT_IGNORE);
   
-  gnutls_dh_set_prime_bits (*session, mc->dh_bits);
+  gnutls_dh_set_prime_bits(*session, mc->dh_bits);
   
   return 0;
 }
@@ -501,6 +507,7 @@
 				      AvahiIfIndex if_index,
 				      mandos_context *mc){
   int ret, tcp_sd;
+  ssize_t sret;
   union { struct sockaddr in; struct sockaddr_in6 in6; } to;
   char *buffer = NULL;
   char *decrypted_buffer;
@@ -512,8 +519,8 @@
   char interface[IF_NAMESIZE];
   gnutls_session_t session;
   
-  ret = init_gnutls_session (mc, &session);
-  if (ret != 0){
+  ret = init_gnutls_session(mc, &session);
+  if(ret != 0){
     return -1;
   }
   
@@ -541,7 +548,7 @@
   /* It would be nice to have a way to detect if we were passed an
      IPv4 address here.   Now we assume an IPv6 address. */
   ret = inet_pton(AF_INET6, ip, &to.in6.sin6_addr);
-  if (ret < 0 ){
+  if(ret < 0 ){
     perror("inet_pton");
     return -1;
   }
@@ -549,7 +556,9 @@
     fprintf(stderr, "Bad address: %s\n", ip);
     return -1;
   }
-  to.in6.sin6_port = htons(port); /* Spurious warning */
+  to.in6.sin6_port = htons(port); /* Spurious warnings from
+				     -Wconversion and
+				     -Wunreachable-code */
   
   to.in6.sin6_scope_id = (uint32_t)if_index;
   
@@ -568,18 +577,18 @@
   }
   
   ret = connect(tcp_sd, &to.in, sizeof(to));
-  if (ret < 0){
+  if(ret < 0){
     perror("connect");
     return -1;
   }
   
   const char *out = mandos_protocol_version;
   written = 0;
-  while (true){
+  while(true){
     size_t out_size = strlen(out);
-    ret = TEMP_FAILURE_RETRY(write(tcp_sd, out + written,
+    ret = (int)TEMP_FAILURE_RETRY(write(tcp_sd, out + written,
 				   out_size - written));
-    if (ret == -1){
+    if(ret == -1){
       perror("write");
       retval = -1;
       goto mandos_end;
@@ -588,7 +597,7 @@
     if(written < out_size){
       continue;
     } else {
-      if (out == mandos_protocol_version){
+      if(out == mandos_protocol_version){
 	written = 0;
 	out = "\r\n";
       } else {
@@ -601,16 +610,16 @@
     fprintf(stderr, "Establishing TLS session with %s\n", ip);
   }
   
-  gnutls_transport_set_ptr (session, (gnutls_transport_ptr_t) tcp_sd);
+  gnutls_transport_set_ptr(session, (gnutls_transport_ptr_t) tcp_sd);
   
   do{
-    ret = gnutls_handshake (session);
+    ret = gnutls_handshake(session);
   } while(ret == GNUTLS_E_AGAIN or ret == GNUTLS_E_INTERRUPTED);
   
-  if (ret != GNUTLS_E_SUCCESS){
+  if(ret != GNUTLS_E_SUCCESS){
     if(debug){
       fprintf(stderr, "*** GnuTLS Handshake failed ***\n");
-      gnutls_perror (ret);
+      gnutls_perror(ret);
     }
     retval = -1;
     goto mandos_end;
@@ -626,29 +635,29 @@
   while(true){
     buffer_capacity = adjustbuffer(&buffer, buffer_length,
 				   buffer_capacity);
-    if (buffer_capacity == 0){
+    if(buffer_capacity == 0){
       perror("adjustbuffer");
       retval = -1;
       goto mandos_end;
     }
     
-    ret = gnutls_record_recv(session, buffer+buffer_length,
-			     BUFFER_SIZE);
-    if (ret == 0){
+    sret = gnutls_record_recv(session, buffer+buffer_length,
+			      BUFFER_SIZE);
+    if(sret == 0){
       break;
     }
-    if (ret < 0){
-      switch(ret){
+    if(sret < 0){
+      switch(sret){
       case GNUTLS_E_INTERRUPTED:
       case GNUTLS_E_AGAIN:
 	break;
       case GNUTLS_E_REHANDSHAKE:
 	do{
-	  ret = gnutls_handshake (session);
+	  ret = gnutls_handshake(session);
 	} while(ret == GNUTLS_E_AGAIN or ret == GNUTLS_E_INTERRUPTED);
-	if (ret < 0){
+	if(ret < 0){
 	  fprintf(stderr, "*** GnuTLS Re-handshake failed ***\n");
-	  gnutls_perror (ret);
+	  gnutls_perror(ret);
 	  retval = -1;
 	  goto mandos_end;
 	}
@@ -657,11 +666,11 @@
 	fprintf(stderr, "Unknown error while reading data from"
 		" encrypted session with Mandos server\n");
 	retval = -1;
-	gnutls_bye (session, GNUTLS_SHUT_RDWR);
+	gnutls_bye(session, GNUTLS_SHUT_RDWR);
 	goto mandos_end;
       }
     } else {
-      buffer_length += (size_t) ret;
+      buffer_length += (size_t) sret;
     }
   }
   
@@ -669,18 +678,18 @@
     fprintf(stderr, "Closing TLS session\n");
   }
   
-  gnutls_bye (session, GNUTLS_SHUT_RDWR);
+  gnutls_bye(session, GNUTLS_SHUT_RDWR);
   
-  if (buffer_length > 0){
+  if(buffer_length > 0){
     decrypted_buffer_size = pgp_packet_decrypt(mc, buffer,
 					       buffer_length,
 					       &decrypted_buffer);
-    if (decrypted_buffer_size >= 0){
+    if(decrypted_buffer_size >= 0){
       written = 0;
       while(written < (size_t) decrypted_buffer_size){
-	ret = (int)fwrite (decrypted_buffer + written, 1,
-			   (size_t)decrypted_buffer_size - written,
-			   stdout);
+	ret = (int)fwrite(decrypted_buffer + written, 1,
+			  (size_t)decrypted_buffer_size - written,
+			  stdout);
 	if(ret == 0 and ferror(stdout)){
 	  if(debug){
 	    fprintf(stderr, "Error writing encrypted data: %s\n",
@@ -703,11 +712,11 @@
   
  mandos_end:
   free(buffer);
-  ret = TEMP_FAILURE_RETRY(close(tcp_sd));
+  ret = (int)TEMP_FAILURE_RETRY(close(tcp_sd));
   if(ret == -1){
     perror("close");
   }
-  gnutls_deinit (session);
+  gnutls_deinit(session);
   return retval;
 }
 
@@ -731,7 +740,7 @@
   /* Called whenever a service has been resolved successfully or
      timed out */
   
-  switch (event) {
+  switch(event) {
   default:
   case AVAHI_RESOLVER_FAILURE:
     fprintf(stderr, "(Avahi Resolver) Failed to resolve service '%s'"
@@ -745,11 +754,11 @@
       avahi_address_snprint(ip, sizeof(ip), address);
       if(debug){
 	fprintf(stderr, "Mandos server \"%s\" found on %s (%s, %"
-		PRIu16 ") on port %d\n", name, host_name, ip,
-		interface, port);
+		PRIdMAX ") on port %" PRIu16 "\n", name, host_name,
+		ip, (intmax_t)interface, port);
       }
       int ret = start_mandos_communication(ip, port, interface, mc);
-      if (ret == 0){
+      if(ret == 0){
 	avahi_simple_poll_quit(mc->simple_poll);
       }
     }
@@ -773,7 +782,7 @@
   /* Called whenever a new services becomes available on the LAN or
      is removed from the LAN */
   
-  switch (event) {
+  switch(event) {
   default:
   case AVAHI_BROWSER_FAILURE:
     
@@ -788,7 +797,7 @@
        the callback function is called the Avahi server will free the
        resolver for us. */
     
-    if (!(avahi_s_service_resolver_new(mc->server, interface,
+    if(!(avahi_s_service_resolver_new(mc->server, interface,
 				       protocol, name, type, domain,
 				       AVAHI_PROTO_INET6, 0,
 				       resolve_callback, mc)))
@@ -812,6 +821,8 @@
     AvahiSServiceBrowser *sb = NULL;
     int error;
     int ret;
+    intmax_t tmpmax;
+    int numchars;
     int exitcode = EXIT_SUCCESS;
     const char *interface = "eth0";
     struct ifreq network;
@@ -863,11 +874,9 @@
 	{ .name = NULL }
       };
       
-      error_t parse_opt (int key, char *arg,
-			 struct argp_state *state) {
-	/* Get the INPUT argument from `argp_parse', which we know is
-	   a pointer to our plugin list pointer. */
-	switch (key) {
+      error_t parse_opt(int key, char *arg,
+			struct argp_state *state) {
+	switch(key) {
 	case 128:		/* --debug */
 	  debug = true;
 	  break;
@@ -884,18 +893,19 @@
 	  pubkey = arg;
 	  break;
 	case 129:		/* --dh-bits */
-	  errno = 0;
-	  mc.dh_bits = (unsigned int) strtol(arg, NULL, 10);
-	  if (errno){
-	    perror("strtol");
+	  ret = sscanf(arg, "%" SCNdMAX "%n", &tmpmax, &numchars);
+	  if(ret < 1 or tmpmax != (typeof(mc.dh_bits))tmpmax
+	     or arg[numchars] != '\0'){
+	    fprintf(stderr, "Bad number of DH bits\n");
 	    exit(EXIT_FAILURE);
 	  }
+	  mc.dh_bits = (typeof(mc.dh_bits))tmpmax;
 	  break;
 	case 130:		/* --priority */
 	  mc.priority = arg;
 	  break;
 	case ARGP_KEY_ARG:
-	  argp_usage (state);
+	  argp_usage(state);
 	case ARGP_KEY_END:
 	  break;
 	default:
@@ -908,8 +918,8 @@
 			   .args_doc = "",
 			   .doc = "Mandos client -- Get and decrypt"
 			   " passwords from a Mandos server" };
-      ret = argp_parse (&argp, argc, argv, 0, 0, NULL);
-      if (ret == ARGP_ERR_UNKNOWN){
+      ret = argp_parse(&argp, argc, argv, 0, 0, NULL);
+      if(ret == ARGP_ERR_UNKNOWN){
 	fprintf(stderr, "Unknown error while parsing arguments\n");
 	exitcode = EXIT_FAILURE;
 	goto end;
@@ -940,7 +950,7 @@
 	  goto end;
 	}
       }
-      ret = TEMP_FAILURE_RETRY(close(sd));
+      ret = (int)TEMP_FAILURE_RETRY(close(sd));
       if(ret == -1){
 	perror("close");
       }
@@ -950,17 +960,17 @@
     gid = getgid();
     
     ret = setuid(uid);
-    if (ret == -1){
+    if(ret == -1){
       perror("setuid");
     }
     
     setgid(gid);
-    if (ret == -1){
+    if(ret == -1){
       perror("setgid");
     }
     
     ret = init_gnutls_global(&mc, pubkey, seckey);
-    if (ret == -1){
+    if(ret == -1){
       fprintf(stderr, "init_gnutls_global failed\n");
       exitcode = EXIT_FAILURE;
       goto end;
@@ -997,13 +1007,15 @@
 	exitcode = EXIT_FAILURE;
 	goto end;
       }
-      errno = 0;
-      uint16_t port = (uint16_t) strtol(address+1, NULL, 10);
-      if(errno){
-	perror("Bad port number");
+      uint16_t port;
+      ret = sscanf(address+1, "%" SCNdMAX "%n", &tmpmax, &numchars);
+      if(ret < 1 or tmpmax != (uint16_t)tmpmax
+	 or address[numchars+1] != '\0'){
+	fprintf(stderr, "Bad port number\n");
 	exitcode = EXIT_FAILURE;
 	goto end;
       }
+      port = (uint16_t)tmpmax;
       *address = '\0';
       address = connect_to;
       ret = start_mandos_communication(address, port, if_index, &mc);
@@ -1015,7 +1027,7 @@
       goto end;
     }
     
-    if (not debug){
+    if(not debug){
       avahi_set_log_function(empty_log);
     }
     
@@ -1024,7 +1036,7 @@
     
     /* Allocate main Avahi loop object */
     mc.simple_poll = avahi_simple_poll_new();
-    if (mc.simple_poll == NULL) {
+    if(mc.simple_poll == NULL) {
         fprintf(stderr, "Avahi: Failed to create simple poll"
 		" object.\n");
 	exitcode = EXIT_FAILURE;
@@ -1050,7 +1062,7 @@
     }
     
     /* Check if creating the Avahi server object succeeded */
-    if (mc.server == NULL) {
+    if(mc.server == NULL) {
         fprintf(stderr, "Failed to create Avahi server: %s\n",
 		avahi_strerror(error));
 	exitcode = EXIT_FAILURE;
@@ -1062,7 +1074,7 @@
 				     AVAHI_PROTO_INET6,
 				     "_mandos._tcp", NULL, 0,
 				     browse_callback, &mc);
-    if (sb == NULL) {
+    if(sb == NULL) {
         fprintf(stderr, "Failed to create service browser: %s\n",
 		avahi_strerror(avahi_server_errno(mc.server)));
 	exitcode = EXIT_FAILURE;
@@ -1071,7 +1083,7 @@
     
     /* Run the main loop */
     
-    if (debug){
+    if(debug){
       fprintf(stderr, "Starting Avahi loop search\n");
     }
     
@@ -1079,23 +1091,23 @@
     
  end:
     
-    if (debug){
+    if(debug){
       fprintf(stderr, "%s exiting\n", argv[0]);
     }
     
     /* Cleanup things */
-    if (sb != NULL)
+    if(sb != NULL)
         avahi_s_service_browser_free(sb);
     
-    if (mc.server != NULL)
+    if(mc.server != NULL)
         avahi_server_free(mc.server);
     
-    if (mc.simple_poll != NULL)
+    if(mc.simple_poll != NULL)
         avahi_simple_poll_free(mc.simple_poll);
     
-    if (gnutls_initalized){
+    if(gnutls_initalized){
       gnutls_certificate_free_credentials(mc.cred);
-      gnutls_global_deinit ();
+      gnutls_global_deinit();
       gnutls_dh_params_deinit(mc.dh_params);
     }
     
@@ -1109,14 +1121,16 @@
       struct dirent *direntry;
       d = opendir(tempdir);
       if(d == NULL){
-	perror("opendir");
+	if(errno != ENOENT){
+	  perror("opendir");
+	}
       } else {
 	while(true){
 	  direntry = readdir(d);
 	  if(direntry == NULL){
 	    break;
 	  }
-	  if (direntry->d_type == DT_REG){
+	  if(direntry->d_type == DT_REG){
 	    char *fullname = NULL;
 	    ret = asprintf(&fullname, "%s/%s", tempdir,
 			   direntry->d_name);
@@ -1135,7 +1149,7 @@
 	closedir(d);
       }
       ret = rmdir(tempdir);
-      if(ret == -1){
+      if(ret == -1 and errno != ENOENT){
 	perror("rmdir");
       }
     }

=== modified file 'plugins.d/password-prompt.c'
--- plugins.d/password-prompt.c	2009-01-04 21:54:55 +0000
+++ plugins.d/password-prompt.c	2009-01-13 04:35:19 +0000
@@ -1,6 +1,6 @@
 /*  -*- coding: utf-8 -*- */
 /*
- * Passprompt - Read a password from the terminal and print it
+ * Password-prompt - Read a password from the terminal and print it
  * 
  * Copyright © 2008,2009 Teddy Hogeborn
  * Copyright © 2008,2009 Björn Påhlsson
@@ -79,10 +79,8 @@
 	.doc = "Debug mode", .group = 3 },
       { .name = NULL }
     };
-  
+    
     error_t parse_opt (int key, char *arg, struct argp_state *state) {
-      /* Get the INPUT argument from `argp_parse', which we know is a
-	 pointer to our plugin list pointer. */
       switch (key) {
       case 'p':
 	prefix = arg;
@@ -91,7 +89,7 @@
 	debug = true;
 	break;
       case ARGP_KEY_ARG:
-	argp_usage (state);
+	argp_usage(state);
 	break;
       case ARGP_KEY_END:
 	break;
@@ -100,26 +98,26 @@
       }
       return 0;
     }
-  
+    
     struct argp argp = { .options = options, .parser = parse_opt,
 			 .args_doc = "",
 			 .doc = "Mandos password-prompt -- Read and"
 			 " output a password" };
-    ret = argp_parse (&argp, argc, argv, 0, 0, NULL);
-    if (ret == ARGP_ERR_UNKNOWN){
+    ret = argp_parse(&argp, argc, argv, 0, 0, NULL);
+    if(ret == ARGP_ERR_UNKNOWN){
       fprintf(stderr, "Unknown error while parsing arguments\n");
       return EXIT_FAILURE;
     }
   }
-    
-  if (debug){
+  
+  if(debug){
     fprintf(stderr, "Starting %s\n", argv[0]);
   }
-  if (debug){
+  if(debug){
     fprintf(stderr, "Storing current terminal attributes\n");
   }
   
-  if (tcgetattr(STDIN_FILENO, &t_old) != 0){
+  if(tcgetattr(STDIN_FILENO, &t_old) != 0){
     perror("tcgetattr");
     return EXIT_FAILURE;
   }
@@ -133,7 +131,7 @@
     perror("sigaction");
     return EXIT_FAILURE;
   }
-  if (old_action.sa_handler != SIG_IGN){
+  if(old_action.sa_handler != SIG_IGN){
     ret = sigaction(SIGINT, &new_action, NULL);
     if(ret == -1){
       perror("sigaction");
@@ -145,7 +143,7 @@
     perror("sigaction");
     return EXIT_FAILURE;
   }
-  if (old_action.sa_handler != SIG_IGN){
+  if(old_action.sa_handler != SIG_IGN){
     ret = sigaction(SIGHUP, &new_action, NULL);
     if(ret == -1){
       perror("sigaction");
@@ -157,7 +155,7 @@
     perror("sigaction");
     return EXIT_FAILURE;
   }
-  if (old_action.sa_handler != SIG_IGN){
+  if(old_action.sa_handler != SIG_IGN){
     ret = sigaction(SIGTERM, &new_action, NULL);
     if(ret == -1){
       perror("sigaction");
@@ -166,22 +164,22 @@
   }
   
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "Removing echo flag from terminal attributes\n");
   }
   
   t_new = t_old;
   t_new.c_lflag &= ~ECHO;
-  if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_new) != 0){
+  if(tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_new) != 0){
     perror("tcsetattr-echo");
     return EXIT_FAILURE;
   }
 
-  if (debug){
+  if(debug){
     fprintf(stderr, "Waiting for input from stdin \n");
   }
   while(true){
-    if (quit_now){
+    if(quit_now){
       if(debug){
 	fprintf(stderr, "Interrupted by signal, exiting.\n");
       }
@@ -213,7 +211,7 @@
       }
     }
     ret = getline(&buffer, &n, stdin);
-    if (ret > 0){
+    if(ret > 0){
       status = EXIT_SUCCESS;
       /* Make n = data size instead of allocated buffer size */
       n = (size_t)ret;
@@ -234,8 +232,8 @@
       }
       break;
     }
-    if (ret < 0){
-      if (errno != EINTR and not feof(stdin)){
+    if(ret < 0){
+      if(errno != EINTR and not feof(stdin)){
 	perror("getline");
 	status = EXIT_FAILURE;
 	break;
@@ -250,17 +248,17 @@
       fprintf(stderr, "getline() returned 0, retrying.\n");
     }
   }
-
+  
   free(buffer);
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "Restoring terminal attributes\n");
   }
-  if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_old) != 0){
+  if(tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_old) != 0){
     perror("tcsetattr+echo");
   }
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "%s is exiting with status %d\n", argv[0],
 	    status);
   }

=== modified file 'plugins.d/splashy.c'
--- plugins.d/splashy.c	2009-01-04 21:54:55 +0000
+++ plugins.d/splashy.c	2009-01-14 14:20:17 +0000
@@ -1,6 +1,6 @@
 /*  -*- coding: utf-8 -*- */
 /*
- * Passprompt - Read a password from splashy and output it
+ * Splashy - Read a password from splashy and output it
  * 
  * Copyright © 2008,2009 Teddy Hogeborn
  * Copyright © 2008,2009 Björn Påhlsson
@@ -30,12 +30,13 @@
 				   SIG_IGN, kill(), SIGKILL */
 #include <stddef.h>		/* NULL */
 #include <stdlib.h>		/* getenv() */
-#include <stdio.h>		/* asprintf(), perror() */
-#include <stdlib.h>		/* EXIT_FAILURE, free(), strtoul(),
+#include <stdio.h>		/* asprintf(), perror(), sscanf() */
+#include <stdlib.h>		/* EXIT_FAILURE, free(),
 				   EXIT_SUCCESS */
 #include <sys/types.h>		/* pid_t, DIR, struct dirent,
 				   ssize_t */
 #include <dirent.h>		/* opendir(), readdir(), closedir() */
+#include <inttypes.h>		/* intmax_t, SCNdMAX */
 #include <sys/stat.h>		/* struct stat, lstat(), S_ISLNK */
 #include <iso646.h>		/* not, or, and */
 #include <unistd.h>		/* readlink(), fork(), execl(),
@@ -97,10 +98,18 @@
     for(struct dirent *proc_ent = readdir(proc_dir);
 	proc_ent != NULL;
 	proc_ent = readdir(proc_dir)){
-      pid_t pid = (pid_t) strtoul(proc_ent->d_name, NULL, 10);
-      if(pid == 0){
-	/* Not a process */
-	continue;
+      pid_t pid;
+      {
+	intmax_t tmpmax;
+	int numchars;
+	ret = sscanf(proc_ent->d_name, "%" SCNdMAX "%n", &tmpmax,
+		     &numchars);
+	if(ret < 1 or tmpmax != (pid_t)tmpmax
+	   or proc_ent->d_name[numchars] != '\0'){
+	  /* Not a process */
+	  continue;
+	}
+	pid = (pid_t)tmpmax;
       }
       /* Find the executable name by doing readlink() on the
 	 /proc/<pid>/exe link */
@@ -120,6 +129,10 @@
 	struct stat exe_stat;
 	ret = lstat(exe_link, &exe_stat);
 	if(ret == -1){
+	  if(errno == ENOENT){
+	    free(exe_link);
+	    continue;
+	  }
 	  perror("lstat");
 	  free(exe_link);
 	  free(prompt);
@@ -263,7 +276,7 @@
     /* Child; will become new splashy process */
     
     /* Make the effective user ID (root) the only user ID instead of
-       the real user ID (mandos) */
+       the real user ID (_mandos) */
     ret = setuid(geteuid());
     if(ret == -1){
       perror("setuid");

=== modified file 'plugins.d/usplash.c'
--- plugins.d/usplash.c	2009-01-04 21:54:55 +0000
+++ plugins.d/usplash.c	2009-01-14 14:20:17 +0000
@@ -1,6 +1,6 @@
 /*  -*- coding: utf-8 -*- */
 /*
- * Passprompt - Read a password from usplash and output it
+ * Usplash - Read a password from usplash and output it
  * 
  * Copyright © 2008,2009 Teddy Hogeborn
  * Copyright © 2008,2009 Björn Påhlsson
@@ -36,17 +36,17 @@
 				   dirent */
 #include <stddef.h>		/* NULL */
 #include <string.h>		/* strlen(), memcmp() */
-#include <stdio.h>		/* asprintf(), perror() */
+#include <stdio.h>		/* asprintf(), perror(), sscanf() */
 #include <unistd.h>		/* close(), write(), readlink(),
 				   read(), STDOUT_FILENO, sleep(),
 				   fork(), setuid(), geteuid(),
 				   setsid(), chdir(), dup2(),
 				   STDERR_FILENO, execv() */
-#include <stdlib.h>		/* free(), EXIT_FAILURE, strtoul(),
-				   realloc(), EXIT_SUCCESS, malloc(),
-				   _exit() */
+#include <stdlib.h>		/* free(), EXIT_FAILURE, realloc(),
+				   EXIT_SUCCESS, malloc(), _exit() */
 #include <stdlib.h>		/* getenv() */
 #include <dirent.h>		/* opendir(), readdir(), closedir() */
+#include <inttypes.h>		/* intmax_t, SCNdMAX */
 #include <sys/stat.h>		/* struct stat, lstat(), S_ISLNK */
 
 sig_atomic_t interrupted_by_signal = 0;
@@ -92,10 +92,11 @@
   }
   
   size_t written = 0;
+  ssize_t sret = 0;
   while(not interrupted_by_signal and written < cmd_line_len){
-    ret = write(fifo_fd, cmd_line + written,
-		cmd_line_len - written);
-    if(ret == -1){
+    sret = write(fifo_fd, cmd_line + written,
+		 cmd_line_len - written);
+    if(sret == -1){
       if(errno != EINTR or interrupted_by_signal){
 	int e = errno;
 	close(fifo_fd);
@@ -106,7 +107,7 @@
 	continue;
       }
     }
-    written += (size_t)ret;
+    written += (size_t)sret;
   }
   free(cmd_line_alloc);
   do{
@@ -169,10 +170,18 @@
     for(struct dirent *proc_ent = readdir(proc_dir);
 	proc_ent != NULL;
 	proc_ent = readdir(proc_dir)){
-      pid_t pid = (pid_t) strtoul(proc_ent->d_name, NULL, 10);
-      if(pid == 0){
-	/* Not a process */
-	continue;
+      pid_t pid;
+      {
+	intmax_t tmpmax;
+	int numchars;
+	ret = sscanf(proc_ent->d_name, "%" SCNdMAX "%n", &tmpmax,
+		     &numchars);
+	if(ret < 1 or tmpmax != (pid_t)tmpmax
+	   or proc_ent->d_name[numchars] != '\0'){
+	  /* Not a process */
+	  continue;
+	}
+	pid = (pid_t)tmpmax;
       }
       /* Find the executable name by doing readlink() on the
 	 /proc/<pid>/exe link */
@@ -192,6 +201,10 @@
 	struct stat exe_stat;
 	ret = lstat(exe_link, &exe_stat);
 	if(ret == -1){
+	  if(errno == ENOENT){
+	    free(exe_link);
+	    continue;
+	  }
 	  perror("lstat");
 	  free(exe_link);
 	  free(prompt);
@@ -207,9 +220,6 @@
 	
 	sret = readlink(exe_link, exe_target, sizeof(exe_target));
 	free(exe_link);
-	if(sret == -1){
-	  continue;
-	}
       }
       if((sret == ((ssize_t)sizeof(exe_target)-1))
 	 and (memcmp(usplash_name, exe_target,
@@ -484,7 +494,7 @@
     /* Child; will become new usplash process */
     
     /* Make the effective user ID (root) the only user ID instead of
-       the real user ID (mandos) */
+       the real user ID (_mandos) */
     ret = setuid(geteuid());
     if(ret == -1){
       perror("setuid");

